# RandomizeBookStack.py (Prototype 0.00)
# Test script for adding Ajax Easypose style ERC to a figure.

# 0.01: Corrected yRot to use yRot min/max, not Twist !
#       Add option to use reduced translations range 
#       Removed xScale and zScale, added 'Cover height-width ratio' instead (to match BookStack4 test26f)
#       Added (very basic) setting of 'BOOK_0_TO_31' in the cover material for all nested stacks so they don't repeat books
# 0.00: Original, upl;oaded to post #94 here https://community.hivewire3d.com/threads/my-rather-infrequent-and-possibly-apocryphal-books-project.4492/page-5#post-232973


import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigure(self, thisfig):
		print "getAnyChildStackFigure()"
		
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:15] == "Four Book Stack":
					print "Adding '" + figname +"' to figlist[]"
					self.figlist.append(figname)
					self.getAnyChildStackFigure(childfig)
					
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the user gets cold feet right at the start
		confirmed = poser.DialogSimple.YesNo("This script randomizes the books in the stack.\n\nDo you wish to continue?")
		if not confirmed:
			confirmed = poser.DialogSimple.YesNo("Do you want to see more information about what this script does?\n\n(This opens a debug window with extra information and terminates the script. You'll probably need to resize the debug window to read the information.)")
			if confirmed:
				# Briefly explain to the user what this script does and give an option toabort straight away
				print "This script randomizes the books in the stack... MORE EXPLANATION TO GO HERE"
			return
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:15] == "Four Book Stack"):
			poser.DialogSimple.MessageBox("This script is only for use with 'Four Book Stack' figures.\n\nScript aborted, no changes made.")
			return
		
		print "Adding '" + fig.Name() +"' to figlist[]"
		self.figlist.append(fig.Name())
		
		self.getAnyChildStackFigure(fig)
		
		# geometries ?
		geomtoo=poser.DialogSimple.YesNo("Select random alternate geometries?")
		
		# morphs+adjustments ?
		adjtoo=poser.DialogSimple.YesNo("Randomize morphs + adjustments ?")
		
		# yrot ?
		fullyrot = 0
		yrottoo=poser.DialogSimple.YesNo("Randomize yRotations ?")
		if yrottoo:
			fullyrot=poser.DialogSimple.YesNo("Full 360 degrees ?\n(If not +-10 will be used)")
		
		# translations ?
		fulltrans = 0
		transtoo=poser.DialogSimple.YesNo("Randomize positions ?")
		if transtoo:
			fulltrans=poser.DialogSimple.YesNo("Use full range for positions ?\n(If not 25% of that will be used)")
		
		# scaling ?
		fullscale = 0
		scalestoo=poser.DialogSimple.YesNo("Randomize book dimensions ?")
		if scalestoo:
			fullscale=poser.DialogSimple.YesNo("Use full range for dimensions ?\n(If not 25% of that will be used)")
		
		# Final confirmation
		confirm1=poser.DialogSimple.YesNo("Ready to randomize?\n(final confirmation)")
		if confirm1:
			
			bookcount = 0.0
			for f in self.figlist:
				fig = poser.Scene().Figure(f)
				print fig.Name()
				st = fig.Material("BookMap64Cover").ShaderTree()
				nodelist = st.Nodes()
				for n in nodelist:
					if n.Name() == "BOOK_0_TO_31":
						print "Set book number to " + str(bookcount)
						n.InputByInternalName("Value_1").SetFloat(bookcount)
						bookcount = bookcount + 4
				# For each of the four books in the figure...
				i = 1
				while i < 5:
					# Get the next book
					bone = "book_" + str(i)
					act = fig.ActorByInternalName(bone)
					print "Randomizing " + bone
									
					# Randomize geometry
					if geomtoo:
						pname = "Book" + str(i)+"Geom"
						p = act.Parameter(pname)
						rand = random.randint(p.MinValue(),p.MaxValue()) #int, can be negative
						p.SetValue(rand)
					
					if adjtoo:
						# Randomize 'Splay'
						p = act.Parameter("Splay")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Oblique'
						p = act.Parameter("Oblique")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Shear'
						p = act.Parameter("Shear")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Plump' - OVERRIDE -1 MINIMUM VALUE, USE 0
						p = act.Parameter("Plump")
						rand = random.uniform(0,p.MaxValue()) #float, can be negative
						p.SetValue(rand)
						
						# Randomize 'Shear'
						p = act.Parameter("Shear")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Twist'
						p = act.Parameter("Twist")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Curl (Paperback only)' - BEST NOT TO DO THIS VIA SCRIPT
						#p = act.Parameter("Curl (Paperback only)")
						#rand = random.uniform(p.MinValue(),p.MaxValue())
						#p.SetValue(rand)
					
					# Randomize 'yRotate'
					if yrottoo:
						pmin=0
						pmax=0
						p = act.Parameter("yRotate")
						if fullyrot:
							pmin=p.MinValue()
							pmax=p.MaxValue()
						else:
							pmin=-10
							pmax=10
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
					
					if transtoo:
						# Randomize 'xTran'
						pmin=0
						pmax=0
						p = act.Parameter("xTran")
						if fulltrans:
							pmin=p.MinValue()
							pmax=p.MaxValue()
						else:
							pmin=0.25*p.MinValue()
							pmax=0.25*p.MaxValue()
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
						
						# Randomize 'zTran'
						pmin=0
						pmax=0
						p = act.Parameter("zTran")
						if fulltrans:
							pmin=p.MinValue()
							pmax=p.MaxValue()
						else:
							pmin=0.25*p.MinValue()
							pmax=0.25*p.MaxValue()
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
					
					if scalestoo:
						# Randomize 'Cover height-width ratio'
						pmin=0
						pmax=0
						p = act.Parameter("Cover height-width ratio")
						if fullscale:
							pmin=p.MinValue()
							pmax=p.MaxValue()
						else:
							pmin=1.675+(0.25*(1.675-p.MinValue()))
							pmax=1.675+(0.25*(1.675-p.MaxValue()))
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
						
						# Randomize 'Scale'
						pmin=0
						pmax=0
						p = act.Parameter("Scale")
						if fullscale:
							pmin=p.MinValue()
							pmax=p.MaxValue()
						else:
							pmin=1+(0.25*(1-p.MinValue()))
							pmax=1+(0.25*(1-p.MaxValue()))
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
						
						# Randomize 'yScale'
						pmin=0
						pmax=0
						p = act.Parameter("yScale")
						if fullscale:
							pmin=p.MinValue()
							pmax=p.MaxValue()
						else:
							pmin=1+(0.25*(1-p.MinValue()))
							pmax=1+(0.25*(1-p.MaxValue()))
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
					
					# Next book if any
					i = i + 1
				
		# Redraw what we've modified	
		poser.Scene().Draw()
			
		# Inform the user that it's all done
		poser.DialogSimple.MessageBox("All done")
			
poser.DialogSimple.MessageBox("Running 3DCheapskates RandomizeBookStack.py script (prototype 0.00)\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
poser.DialogSimple.MessageBox("3DCheapskates RandomizeBookStack.py script (prototype 0.00) all done.")